@@ -3,9 +3,9 @@
  *
  * TOMOYO Linux's utilities.
  *
- * Copyright (C) 2005-2011  NTT DATA CORPORATION
+ * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 2.5.0+   2011/10/25
+ * Version: 2.5.0+   2013/12/01
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License v2 as published by the
@@ -185,7 +185,7 @@ static unsigned char revalidate_path(const char *path)
 	return type;
 }
 
-/* File handle to /etc/tomoyo/current/policy/exception_policy.conf . */
+/* File handle to /etc/tomoyo/policy/current/exception_policy.conf . */
 static FILE *filp = NULL;
 
 /**
@@ -305,6 +305,22 @@ static void scan_init_scripts(void)
 	free(namelist);
 }
 
+/**
+ * make_systemd_exceptions - Exceptions specific to systemd
+ *
+ * Returns nothing.
+ */
+static void make_systemd_exceptions(void)
+{
+	/* allow systemd to re-execute itself */
+	if (access("/lib/systemd/systemd", X_OK) == 0)
+		fprintf(filp, "keep_domain /lib/systemd/systemd from "
+			"<kernel> /sbin/init\n");
+	if (access("/usr/lib/systemd/systemd", X_OK) == 0)
+		fprintf(filp, "keep_domain /usr/lib/systemd/systemd from "
+			"<kernel> /sbin/init\n");
+}
+
 /**
  * make_init_scripts_as_aggregators - Use realpath for startup/shutdown scripts in /etc/ directory.
  *
@@ -460,6 +476,22 @@ static void make_ldconfig_readable_files(void)
 		"/usr/lib/tls/i686/", "/usr/lib/tls/i686/cmov/",
 		"/usr/lib/sse2/", "/usr/X11R6/lib/", "/usr/lib32/",
 		"/usr/lib64/", "/lib64/", "/lib64/tls/",
+		"/usr/lib/x86_64-linux-gnu/", "/lib/x86_64-linux-gnu/",
+		"/usr/lib/i386-linux-gnu/", "/lib/i386-linux-gnu/",
+		"/usr/lib/arm-linux-gnueabihf/", "/lib/arm-linux-gnueabihf/",
+		"/usr/lib/arm-linux-gnueabi/", "/lib/arm-linux-gnueabi/",
+		"/usr/lib/aarch64-linux-gnu/", "/lib/aarch64-linux-gnu/",
+		"/usr/lib/ia64-linux-gnu/", "/lib/ia64-linux-gnu/",
+		"/usr/lib/mips-linux-gnu/", "/lib/mips-linux-gnu/",
+		"/usr/lib/mipsel-linux-gnu/", "/lib/mipsel-linux-gnu/",
+		"/usr/lib/powerpc-linux-gnu/", "/lib/powerpc-linux-gnu/",
+		"/usr/lib/ppc64-linux-gnu/", "/lib/ppc64-linux-gnu/",
+		"/usr/lib/s390-linux-gnu/", "/lib/s390-linux-gnu/",
+		"/usr/lib/s390x-linux-gnu/", "/lib/s390x-linux-gnu/",
+		"/usr/lib/sh4-linux-gnu/", "/lib/sh4-linux-gnu/",
+		"/usr/lib/sparc-linux-gnu/", "/lib/sparc-linux-gnu/",
+		"/usr/lib/sparc64-linux-gnu/", "/lib/sparc64-linux-gnu/",
+		"/usr/lib/x86_64-linux-gnux32/", "/lib/x86_64-linux-gnux32/",
 	};
 	int i;
 	FILE *fp = !access("/sbin/ldconfig", X_OK) ||
@@ -814,6 +846,7 @@ static void make_exception_policy(void)
 	make_init_dir_as_initializers();
 	make_initializers();
 	make_init_scripts_as_aggregators();
+	make_systemd_exceptions();
 	/* Some applications do execve("/proc/self/exe"). */
 	fprintf(filp, "aggregator proc:/self/exe /proc/self/exe\n");
 	close_file(filp, chdir_policy(), "exception_policy.tmp",
@@ -859,8 +892,6 @@ static const char *reject_log = "yes";
 static unsigned int max_audit_log = 1024;
 /* How many ACL entries to add automatically by learning mode? */
 static unsigned int max_learning_entry = 2048;
-/* How long should we carry sleep penalty? */
-static unsigned int enforcing_penalty = 0;
 
 /**
  * make_profile - Make /etc/tomoyo/policy/current/profile.conf .
@@ -885,29 +916,25 @@ static void make_profile(void)
 		file_only = "::file";
 	fprintf(fp, "PROFILE_VERSION=20110903\n");
 	fprintf(fp, "0-COMMENT=-----Disabled Mode-----\n"
-		"0-PREFERENCE={ max_audit_log=%u max_learning_entry=%u "
-		"enforcing_penalty=%u }\n"
+		"0-PREFERENCE={ max_audit_log=%u max_learning_entry=%u }\n"
 		"0-CONFIG%s={ mode=disabled grant_log=%s reject_log=%s }\n",
-		max_audit_log, max_learning_entry, enforcing_penalty,
-		file_only, grant_log, reject_log);
+		max_audit_log, max_learning_entry, file_only, grant_log,
+		reject_log);
 	fprintf(fp, "1-COMMENT=-----Learning Mode-----\n"
-		"1-PREFERENCE={ max_audit_log=%u max_learning_entry=%u "
-		"enforcing_penalty=%u }\n"
+		"1-PREFERENCE={ max_audit_log=%u max_learning_entry=%u }\n"
 		"1-CONFIG%s={ mode=learning grant_log=%s reject_log=%s }\n",
-		max_audit_log, max_learning_entry, enforcing_penalty,
-		file_only, grant_log, reject_log);
+		max_audit_log, max_learning_entry, file_only, grant_log,
+		reject_log);
 	fprintf(fp, "2-COMMENT=-----Permissive Mode-----\n"
-		"2-PREFERENCE={ max_audit_log=%u max_learning_entry=%u "
-		"enforcing_penalty=%u }\n"
+		"2-PREFERENCE={ max_audit_log=%u max_learning_entry=%u }\n"
 		"2-CONFIG%s={ mode=permissive grant_log=%s reject_log=%s }\n",
-		max_audit_log, max_learning_entry, enforcing_penalty,
-		file_only, grant_log, reject_log);
+		max_audit_log, max_learning_entry, file_only, grant_log,
+		reject_log);
 	fprintf(fp, "3-COMMENT=-----Enforcing Mode-----\n"
-		"3-PREFERENCE={ max_audit_log=%u max_learning_entry=%u "
-		"enforcing_penalty=%u }\n"
+		"3-PREFERENCE={ max_audit_log=%u max_learning_entry=%u }\n"
 		"3-CONFIG%s={ mode=enforcing grant_log=%s reject_log=%s }\n",
-		max_audit_log, max_learning_entry, enforcing_penalty,
-		file_only, grant_log, reject_log);
+		max_audit_log, max_learning_entry, file_only, grant_log,
+		reject_log);
 	close_file(fp, 1, "profile.tmp", "profile.conf");
 }
 
@@ -1532,15 +1559,15 @@ static const char notifyd_data[] =
 "#\n"
 "# action_to_take is a command line you want to use for notification.\n"
 "# The command specified by this parameter must read the policy violation\n"
-"# notifycation from standard input. For example, mail, curl and xmessage\n"
+"# notification from standard input. For example, mail, curl and xmessage\n"
 "# commands can read from standard input.\n"
 "# This parameter is passed to execve(). Thus, please use a wrapper program\n"
 "# if you need shell processing (e.g. wildcard expansion, environment\n"
 "# variables).\n"
 "#\n"
 "# minimal_interval is grace time in second before re-notifying the next\n"
-"# occurrence of policy violation. You can specify 60 to limit notifycation\n"
-"# to once per a minute, 3600 to limit notifycation to once per an hour.\n"
+"# occurrence of policy violation. You can specify 60 to limit notification\n"
+"# to once per a minute, 3600 to limit notification to once per an hour.\n"
 "# You can specify 0 to unlimit, but notifying of every policy violation\n"
 "# events (e.g. sending a mail) might annoy you because policy violation\n"
 "# can occur in clusters if once occurred.\n"
@@ -1620,9 +1647,7 @@ int main(int argc, char *argv[])
 			reject_log = arg + 11;
 		} else if (!sscanf(arg, "max_audit_log=%u", &max_audit_log) &&
 			   !sscanf(arg, "max_learning_entry=%u",
-				   &max_learning_entry) &&
-			   !sscanf(arg, "enforcing_penalty=%u",
-				   &enforcing_penalty)) {
+				   &max_learning_entry)) {
 			fprintf(stderr, "Unknown option: '%s'\n", argv[i]);
 			return 1;
 		}
